@@ -1,15 +1,13 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2020 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2020-2021 -- leonerd@leonerd.org.uk
 
-package XS::Parse::Sublike::Builder;
+package XS::Parse::Sublike::Builder 0.12;
 
-use strict;
+use v5.14;
 use warnings;
 
-our $VERSION = '0.09';
-
 =head1 NAME
 
 C<XS::Parse::Sublike::Builder> - build-time support for C<XS::Parse::Sublike>
@@ -122,7 +120,7 @@ __DATA__
 #ifndef __XS_PARSE_SUBLIKE_H__
 #define __XS_PARSE_SUBLIKE_H__
 
-#define XSPARSESUBLIKE_ABI_VERSION 2
+#define XSPARSESUBLIKE_ABI_VERSION 3
 
 struct XSParseSublikeContext {
   SV *name;  /* may be NULL for anon subs */
@@ -149,44 +147,44 @@ struct XSParseSublikeHooks {
   U16  flags;
   U8   require_parts;
   U8   skip_parts;
-  bool (*permit)         (pTHX);
-  void (*pre_subparse)   (pTHX_ struct XSParseSublikeContext *ctx);
-  void (*post_blockstart)(pTHX_ struct XSParseSublikeContext *ctx);
-  void (*pre_blockend)   (pTHX_ struct XSParseSublikeContext *ctx);
-  void (*post_newcv)     (pTHX_ struct XSParseSublikeContext *ctx);
+  bool (*permit)         (pTHX_ void *hookdata);
+  void (*pre_subparse)   (pTHX_ struct XSParseSublikeContext *ctx, void *hookdata);
+  void (*post_blockstart)(pTHX_ struct XSParseSublikeContext *ctx, void *hookdata);
+  void (*pre_blockend)   (pTHX_ struct XSParseSublikeContext *ctx, void *hookdata);
+  void (*post_newcv)     (pTHX_ struct XSParseSublikeContext *ctx, void *hookdata);
 
   /* if flags & XS_PARSE_SUBLIKE_FLAG_FILTERATTRS */
-  bool (*filter_attr)    (pTHX_ struct XSParseSublikeContext *ctx, SV *attr, SV *val);
+  bool (*filter_attr)    (pTHX_ struct XSParseSublikeContext *ctx, SV *attr, SV *val, void *hookdata);
 };
 
-static int (*parse_func)(pTHX_ const struct XSParseSublikeHooks *hooks, OP **op_ptr);
-#define xs_parse_sublike(hooks, op_ptr)  S_xs_parse_sublike(aTHX_ hooks, op_ptr)
-static int S_xs_parse_sublike(pTHX_ const struct XSParseSublikeHooks *hooks, OP **op_ptr)
+static int (*parse_xs_parse_sublike_func)(pTHX_ const struct XSParseSublikeHooks *hooks, void *hookdata, OP **op_ptr);
+#define xs_parse_sublike(hooks, hookdata, op_ptr)  S_xs_parse_sublike(aTHX_ hooks, hookdata, op_ptr)
+static int S_xs_parse_sublike(pTHX_ const struct XSParseSublikeHooks *hooks, void *hookdata, OP **op_ptr)
 {
-  if(!parse_func)
+  if(!parse_xs_parse_sublike_func)
     croak("Must call boot_xs_parse_sublike() first");
 
-  return (*parse_func)(aTHX_ hooks, op_ptr);
+  return (*parse_xs_parse_sublike_func)(aTHX_ hooks, hookdata, op_ptr);
 }
 
-static void (*register_func)(pTHX_ const char *kw, const struct XSParseSublikeHooks *hooks);
-#define register_xs_parse_sublike(kw, hooks) S_register_xs_parse_sublike(aTHX_ kw, hooks)
-static void S_register_xs_parse_sublike(pTHX_ const char *kw, const struct XSParseSublikeHooks *hooks)
+static void (*register_xs_parse_sublike_func)(pTHX_ const char *kw, const struct XSParseSublikeHooks *hooks, void *hookdata);
+#define register_xs_parse_sublike(kw, hooks, hookdata) S_register_xs_parse_sublike(aTHX_ kw, hooks, hookdata)
+static void S_register_xs_parse_sublike(pTHX_ const char *kw, const struct XSParseSublikeHooks *hooks, void *hookdata)
 {
-  if(!register_func)
+  if(!register_xs_parse_sublike_func)
     croak("Must call boot_xs_parse_sublike() first");
 
-  return (*register_func)(aTHX_ kw, hooks);
+  return (*register_xs_parse_sublike_func)(aTHX_ kw, hooks, hookdata);
 }
 
-static int (*parseany_func)(pTHX_ const struct XSParseSublikeHooks *hooks, OP **op_ptr);
-#define xs_parse_sublike_any(hooks, op_ptr)  S_xs_parse_sublike_any(aTHX_ hooks, op_ptr)
-static int S_xs_parse_sublike_any(pTHX_ const struct XSParseSublikeHooks *hooks, OP **op_ptr)
+static int (*parseany_xs_parse_sublike_func)(pTHX_ const struct XSParseSublikeHooks *hooks, void *hookdata, OP **op_ptr);
+#define xs_parse_sublike_any(hooks, hookdata, op_ptr)  S_xs_parse_sublike_any(aTHX_ hooks, hookdata, op_ptr)
+static int S_xs_parse_sublike_any(pTHX_ const struct XSParseSublikeHooks *hooks, void *hookdata, OP **op_ptr)
 {
-  if(!parseany_func)
+  if(!parseany_xs_parse_sublike_func)
     croak("Must call boot_xs_parse_sublike() first");
 
-  return (*parseany_func)(aTHX_ hooks, op_ptr);
+  return (*parseany_xs_parse_sublike_func)(aTHX_ hooks, hookdata, op_ptr);
 }
 
 #define boot_xs_parse_sublike(ver) S_boot_xs_parse_sublike(aTHX_ ver)
@@ -200,13 +198,13 @@ static void S_boot_xs_parse_sublike(pTHX_ double ver) {
     croak("XS::Parse::Sublike ABI version mismatch - library provides %d, compiled for %d",
         abi_version, XSPARSESUBLIKE_ABI_VERSION);
 
-  parse_func = INT2PTR(int (*)(pTHX_ const struct XSParseSublikeHooks *, OP**),
+  parse_xs_parse_sublike_func = INT2PTR(int (*)(pTHX_ const struct XSParseSublikeHooks *, void *, OP**),
       SvUV(get_sv("XS::Parse::Sublike::PARSE", 0)));
 
-  register_func = INT2PTR(void (*)(pTHX_ const char *, const struct XSParseSublikeHooks *),
+  register_xs_parse_sublike_func = INT2PTR(void (*)(pTHX_ const char *, const struct XSParseSublikeHooks *, void *),
       SvUV(get_sv("XS::Parse::Sublike::REGISTER", 0)));
 
-  parseany_func = INT2PTR(int (*)(pTHX_ const struct XSParseSublikeHooks *, OP**),
+  parseany_xs_parse_sublike_func = INT2PTR(int (*)(pTHX_ const struct XSParseSublikeHooks *, void *, OP**),
       SvUV(get_sv("XS::Parse::Sublike::PARSEANY", 0)));
 }
 
